package dk.sosi.sts.nemlogin.uuid2cpr;

import dk.nemlog_in.lookupservice.LookupService;
import dk.nemlog_in.lookupservice.LookupService_Service;
import dk.sosi.sts.nemlogin.sts.client.NemLoginStsClient;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.message.Message;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.CprBySubjectSerialNumberResponse;

import javax.xml.namespace.QName;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class Uuid2CprClient {
    private Uuid2CprProperties properties;

    // We want to reuse the jax-ws client proxies, since tokens from the sts are cached there.
    // See https://cxf.apache.org/faq.html#FAQ%2DAreJAX%2DWSclientproxiesthreadsafe%3F for details on thread safety
    // for these proxies - for this use case, it should be ok to reuse the instance in multiple threads.
    private LookupService lookupService;

    public Uuid2CprClient(Uuid2CprProperties properties) {
        this.properties = properties;
    }

    public Optional<String> lookupCpr(String subjectSerialNumber) {
        LookupService client = getClient();

        CprBySubjectSerialNumberResponse response = client.subjectSerialNumberCpr(subjectSerialNumber);

        if(response.getCpr() == null || response.getCpr().getValue() == null || response.getCpr().getValue().isEmpty()) {
            return Optional.empty();
        }

        return Optional.of(response.getCpr().getValue());
    }

    private LookupService getClient() {
        if(lookupService == null) {
            initializeClient();
        }
        return lookupService;
    }

    private void initializeClient() {
        LookupService client = new LookupService_Service().getSoapBindingLookupService();

        // Set up the client
        Client proxy = ClientProxy.getClient(client);

        proxy.getInInterceptors().add(new LoggingInInterceptor());
        proxy.getOutInterceptors().add(new LoggingOutInterceptor());

        proxy.getRequestContext().put("ws-security.sts.applies-to", properties.getAppliesTo());
        proxy.getRequestContext().put("ws-security.signature.username", properties.getSignatureUsername());
        proxy.getRequestContext().put("ws-security.signature.password", properties.getSignaturePassword());
        proxy.getRequestContext().put("ws-security.signature.properties", properties.getSignaturePropertiesFile());
        proxy.getRequestContext().put("ws-security.asymmetric.signature.algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
        proxy.getRequestContext().put("ws-security.is-bsp-compliant", "false");
        proxy.getRequestContext().put(Message.ENDPOINT_ADDRESS, properties.getLookupServiceUrl());

        // Set up the sts client
        NemLoginStsClient stsClient = new NemLoginStsClient(proxy.getBus());

        Map<String, Object> stsProperties = new HashMap<>();
        stsProperties.put("ws-security.signature.username", properties.getSignatureUsername());
        stsProperties.put("ws-security.signature.password", properties.getSignaturePassword());
        stsProperties.put("ws-security.is-bsp-compliant", "false");
        stsProperties.put("ws-security.signature.properties", properties.getSignaturePropertiesFile());
        stsProperties.put("ws-security.encryption.username", properties.getEncryptionUsername());
        stsProperties.put("ws-security.encryption.properties", properties.getEncryptionPropertiesFile());
        stsProperties.put("ws-security.asymmetric.signature.algorithm", "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");
        stsClient.setProperties(stsProperties);
        stsClient.setWsdlLocation("classpath:sts.wsdl");
        stsClient.setServiceQName(new QName("http://docs.oasis-open.org/ws-sx/ws-trust/200512/", "SecurityTokenService"));
        stsClient.setEndpointQName(new QName("http://docs.oasis-open.org/ws-sx/ws-trust/200512/", "STS_Port"));
        stsClient.setSendRenewing(false);
        stsClient.setSendKeyType(false);
        stsClient.setRequiresEntropy(false);
        stsClient.setTokenType("http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0");
        stsClient.getRequestContext().put(Message.ENDPOINT_ADDRESS, properties.getSecurityTokenServiceUrl());

        stsClient.getInInterceptors().add(new LoggingInInterceptor());
        stsClient.getOutInterceptors().add(new LoggingOutInterceptor());

        proxy.getRequestContext().put("ws-security.sts.client", stsClient);

        lookupService = client;
    }
}
