package dk.sosi.sts.nemlogin.uuid2cpr;

import org.junit.Before;
import org.junit.Test;

import javax.xml.ws.soap.SOAPFaultException;
import java.util.Optional;

import static org.junit.Assert.*;

/**
 * These tests correspond to section 4 of https://jira.nspop.dk/secure/attachment/13536/13536_Integration+med+UUID2CPR+opslagsservice+v01.pdf
 * Tests regarding token caching and expiry have been deliberately omitted - they are cumbersome to write, and would essentially just test that
 * the CXF framework handles token caching correctly (which we assume it does).
 */
public class Uuid2CprClientIT {
    private Uuid2CprClient subject;

    @Before
    public void setup() {
        Uuid2CprProperties properties = new Uuid2CprProperties();
        properties.setAppliesTo("https://saml.wsp.lookupservice.devtest4-nemlog-in.dk");
        properties.setSignatureUsername("lookup");
        properties.setSignaturePassword("Test1234");
        properties.setSignaturePropertiesFile("client.properties");
        properties.setEncryptionUsername("sts");
        properties.setEncryptionPropertiesFile("sts.properties");

        properties.setLookupServiceUrl("https://lookupservice.devtest4-nemlog-in.dk/LookupService.svc");
        properties.setSecurityTokenServiceUrl("https://SecureTokenService.devtest4-nemlog-in.dk/SecurityTokenService.svc");
        properties.setTokenTimeToLive(1234);

        subject = new Uuid2CprClient(properties);
    }

    @Test
    public void lookupCpr_globalUuid_success() {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";

        // Act
        Optional<String> cpr = subject.lookupCpr(subjectSerialNumber);

        // Assert
        assertTrue(cpr.isPresent());
        assertEquals("2501879875", cpr.get());
    }

    @Test
    public void lookupCpr_globalUuid_uuidUnknown_failure() {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";

        // Act
        Optional<String> cpr = subject.lookupCpr(subjectSerialNumber);

        // Assert
        assertFalse(cpr.isPresent());
    }

    @Test
    public void lookupCpr_certificateSpecificUuid_success() {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:edc04d69-f2d4-4e63-b9ed-593f06c1ff5d";

        // Act
        Optional<String> cpr = subject.lookupCpr(subjectSerialNumber);

        // Assert
        assertTrue(cpr.isPresent());
        assertEquals("2501879875", cpr.get());
    }

    @Test
    public void lookupCpr_certificateSpecificUuid_uuidUnknown_failure() {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:00000000-0000-0000-0000-000000000001";

        // Act
        Optional<String> cpr = subject.lookupCpr(subjectSerialNumber);

        // Assert
        assertFalse(cpr.isPresent());
    }

    @Test(expected = SOAPFaultException.class)
    public void lookupCpr_malformedSubjectSerialNumber_failure() {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:not-even-remotely-correct-input";

        // Act
        Optional<String> cpr = subject.lookupCpr(subjectSerialNumber);

        // Assert
    }
}